﻿using UnityEngine;
using System.Collections;

public class Movements : MonoBehaviour {

	public float m_horizontalSpeed = 5.0f;
	public float m_horizontalAcceleration = 10.0f;
	public float m_horizontalDeceleration = 20.0f;
	public float m_inAirDecelerationFactor = 0.25f;
	public float m_verticalSpeed = 5.0f;

	public LayerMask m_groundLayers;
	public Transform m_reflection = null;

	private float m_currentHorizontalSpeed = 0.0f;
	private bool m_underTheSurface = false;
	private bool m_grounded = false;
	private BoxCollider2D m_collider = null;

	private bool m_jumpInput = false;
	private bool m_flipInput = false;
	private bool m_leftInput = false;
	private bool m_rightInput = false;

	void Start( ) {
		m_collider = collider2D as BoxCollider2D;
	}

	void Update( ) {

		// Hack : if already a jump or flip input, don't reset those flags (There can be 2 Update calls without a FixedUpdate call).
		if ( !m_jumpInput ) {
			m_jumpInput = Input.GetKeyDown( KeyCode.UpArrow );
		}

		if ( !m_flipInput ) {
			m_flipInput = Input.GetKeyDown( KeyCode.Space );
		}

		m_leftInput = Input.GetKey( KeyCode.LeftArrow );
		m_rightInput = Input.GetKey( KeyCode.RightArrow );

		if ( transform.position.y < 0.0f && m_reflection.renderer.enabled ) {
			m_reflection.renderer.enabled = false;
		} else if ( transform.position.y > 0.0f && !m_reflection.renderer.enabled ) {
			m_reflection.renderer.enabled = true;
		}

		m_reflection.transform.position = new Vector3( transform.position.x, -transform.position.y, transform.position.z );
	}

	void FixedUpdate( ) {

		m_grounded = false;

		if ( Physics2D.Raycast( transform.position + new Vector3( m_collider.size.x * 0.49f, 0.0f ), ( ( m_underTheSurface ) ? 1 : -1 ) * Vector2.up, m_collider.size.y * 0.6f, m_groundLayers )
				|| Physics2D.Raycast( transform.position - new Vector3( m_collider.size.x * 0.49f, 0.0f ), ( ( m_underTheSurface ) ? 1 : -1 ) * Vector2.up, m_collider.size.y * 0.6f, m_groundLayers ) ) {

			m_grounded = true;
		}

		if ( m_leftInput ) {

			if ( Physics2D.Raycast( transform.position + new Vector3( 0.0f, m_collider.size.y * 0.5f ), -Vector2.right, m_collider.size.x * 0.55f, m_groundLayers )
				|| Physics2D.Raycast( transform.position - new Vector3( 0.0f, m_collider.size.y * 0.5f ), -Vector2.right, m_collider.size.x * 0.55f, m_groundLayers ) ) {

				m_currentHorizontalSpeed = 0.0f;

			} else {

				float factor = 1.0f;

				if ( m_currentHorizontalSpeed > 0.0f ) {
					factor = 3.0f;
				}

				m_currentHorizontalSpeed -= m_horizontalAcceleration * factor * Time.deltaTime;
			}

		} else if ( m_rightInput ) {

			if ( Physics2D.Raycast( transform.position + new Vector3( 0.0f, m_collider.size.y * 0.5f ), Vector2.right, m_collider.size.x * 0.55f, m_groundLayers )
				|| Physics2D.Raycast( transform.position - new Vector3( 0.0f, m_collider.size.y * 0.5f ), Vector2.right, m_collider.size.x * 0.55f, m_groundLayers ) ) {

				m_currentHorizontalSpeed = 0.0f;

			} else {

				float factor = 1.0f;

				if ( m_currentHorizontalSpeed < 0.0f ) {
					factor = 3.0f;
				}

				m_currentHorizontalSpeed += m_horizontalAcceleration * factor * Time.deltaTime;
			}

		} else {

			float factor = 1.0f;

			if ( !m_grounded ) {
				factor = m_inAirDecelerationFactor;
			}

			float oldHorizontalSpeed = m_currentHorizontalSpeed;
			
			if ( m_currentHorizontalSpeed > 0 ) {
				m_currentHorizontalSpeed -= m_horizontalDeceleration * factor * Time.deltaTime;
			} else if ( m_currentHorizontalSpeed < 0 ) {
				m_currentHorizontalSpeed += m_horizontalDeceleration * factor * Time.deltaTime;
			}

			if ( Mathf.Sign( oldHorizontalSpeed ) != Mathf.Sign( m_currentHorizontalSpeed ) ) {
				m_currentHorizontalSpeed = 0.0f;
			}
		}

		if ( m_currentHorizontalSpeed > m_horizontalSpeed ) {
			m_currentHorizontalSpeed = m_horizontalSpeed;
		} else if ( m_currentHorizontalSpeed < -m_horizontalSpeed ) {
			m_currentHorizontalSpeed = -m_horizontalSpeed;
		}

		float verticalSpeed = rigidbody2D.velocity.y;

		if ( m_jumpInput ) {

			m_jumpInput = false;

			if ( m_grounded ) {

				verticalSpeed = ( m_underTheSurface ) ? -m_verticalSpeed : m_verticalSpeed;
			}
		}

		if ( m_flipInput ) {

			m_flipInput = false;

			Vector2 pointA = new Vector2( transform.position.x, -transform.position.y ) - ( m_collider.size * 0.45f );
			Vector2 pointB = new Vector2( transform.position.x, -transform.position.y ) + ( m_collider.size * 0.45f );

			if ( !Physics2D.OverlapArea( pointA, pointB ) ) {

				m_underTheSurface = !m_underTheSurface;
				verticalSpeed = -verticalSpeed;
				Physics2D.gravity = new Vector2( Physics2D.gravity.x, -Physics2D.gravity.y );
				transform.position = new Vector3( transform.position.x, -transform.position.y, transform.position.z );
			}
		}

		rigidbody2D.velocity = new Vector2( m_currentHorizontalSpeed, verticalSpeed );
	}
}
